export const StandardTextStyle = "16px sans-serif";
export const StandardTextHeight = 20;
export const StandardVerticalMargin = 10;
export const StandardLeftPadding = 5;
export const PanelBG = "#c4c2c2";
export const ControlBG = "#dcdcdc";
export const TextColor = "#4d4d4d";
export const SelectedColor = "#7bc50b";
export const SelectedTextColor = "#eceaea";
export const STD_STYLE: {
    PADDING: {
        LEFT: number;
    };
    CONTROL: {
        BACKGROUND: {
            COLOR: string;
        };
        BORDER: {
            COLOR: string;
        };
        TEXT: {
            COLOR: string;
        };
    };
    PANEL: {
        BACKGROUND: {
            COLOR: string;
        };
        BORDER: {
            COLOR: string;
        };
        TEXT: {
            COLOR: string;
        };
    };
    ACTIVE: {
        BACKGROUND: {
            COLOR: string;
        };
        TEXT: {
            COLOR: string;
        };
    };
    SELECTED: {
        BACKGROUND: {
            COLOR: string;
        };
        TEXT: {
            COLOR: string;
        };
    };
};
declare class Insets {
    left: number;
    right: number;
    top: number;
    bottom: number;
    constructor(top: any, right: any, bottom: any, left: any);
}
export type Style = {
    background_color: string;
    border_color: string;
    text_color: string;
    padding: Insets;
};
export const name_styles: Map<string, Style>;
export const selected_styles: Map<string, Style>;
export const active_styles: Map<string, Style>;
export const hover_styles: Map<string, Style>;
export function calculate_style(name: string, selected: boolean, active?: boolean, enabled?: boolean, hover?: boolean): Style;
export interface Palette {
    get_color_palette(): string[];
}
export class Sprite {
    id: string;
    name: string;
    w: number;
    h: number;
    data: number[];
    _img: HTMLCanvasElement;
    constructor(id: string, name: string, w: number, h: number, palette: Palette);
    forEachPixel(cb: (val: any, i: number, j: number) => void): void;
    set_pixel(x: number, y: number, color: any): void;
    sync(): void;
    get_pixel(x: number, y: number): any;
    toJsonObj(): object;
}
export class Sheet {
    sprites: Sprite[];
    name: string;
    constructor(id: string, name: string);
    add(sprite: Sprite): void;
    toJsonObj(): {
        clazz: string;
        id: string;
        name: string;
        sprites: object[];
    };
}
export class Tilemap {
    id: string;
    name: string;
    w: number;
    h: number;
    data: number[];
    constructor(id: any, name: any, w: any, h: any);
    forEachPixel(cb: (val: any, i: number, j: number) => void): void;
    expand_width(number: number): void;
    set_pixel(x: number, y: number, color: any): void;
    get_pixel(x: number, y: number): any;
    toJsonObj(): {
        clazz: string;
        id: string;
        name: string;
        w: number;
        h: number;
        data: number[];
    };
}
type GlyphMeta = {
    codepoint: number;
    left: number;
    right: number;
    baseline: number;
};
export class SpriteGlyph extends Sprite {
    meta: GlyphMeta;
    constructor(id: any, name: any, w: any, h: any, palette: any);
    sync(): void;
    toJsonObj(): object;
}
export class SpriteFont {
    glyphs: SpriteGlyph[];
    id: string;
    name: string;
    constructor(id: any, name: any);
    toJsonObj(): {
        clazz: string;
        id: string;
        name: string;
        glyphs: object[];
    };
    add(spriteGlyph: SpriteGlyph): void;
}
export interface AssetsDoc extends Palette {
    find_sheet(name: String): Sheet;
    find_sprite(sheet_name: String, sprite_name: String): Sprite;
}
export function load_assets_from_json(data: any): AssetsDoc;
export class MouseInputService {
    debug: boolean;
    constructor(surface: SurfaceContext);
    trigger_mouse_down(position: Point, button: number): void;
    trigger_mouse_move(position: Point, button: number): void;
    trigger_mouse_up(position: Point, button: number): void;
    trigger_scroll(position: Point, delta: Point): void;
}
export class KeyboardInputService {
    debug: boolean;
    constructor(surface: SurfaceContext);
    dispatch_keyboard_focus_change(old_focus: View, new_focus: View): void;
    propagateKeyboardEvent(evt: KeyboardEvent, path: View[]): void;
    trigger_key_down(key: string, code: string, modifiers: Modifiers): void;
    trigger_key_up(key: string, code: string, modifiers: Modifiers): void;
}
export const LOGICAL_KEYBOARD_CODE: {
    KEY_A: string;
    KEY_B: string;
    KEY_C: string;
    KEY_D: string;
    KEY_E: string;
    KEY_F: string;
    KEY_G: string;
    KEY_H: string;
    KEY_I: string;
    KEY_J: string;
    KEY_K: string;
    KEY_L: string;
    KEY_M: string;
    KEY_N: string;
    KEY_O: string;
    KEY_P: string;
    KEY_Q: string;
    KEY_R: string;
    KEY_S: string;
    KEY_T: string;
    KEY_U: string;
    KEY_V: string;
    KEY_W: string;
    KEY_X: string;
    KEY_Y: string;
    KEY_Z: string;
    DIGIT_0: string;
    DIGIT_1: string;
    DIGIT_2: string;
    DIGIT_3: string;
    DIGIT_4: string;
    DIGIT_5: string;
    DIGIT_6: string;
    DIGIT_7: string;
    DIGIT_8: string;
    DIGIT_9: string;
    ARROW_LEFT: string;
    ARROW_RIGHT: string;
    ARROW_UP: string;
    ARROW_DOWN: string;
    SHIFT_LEFT: string;
    SHIFT_RIGHT: string;
    CONTROL_LEFT: string;
    CONTROL_RIGHT: string;
    ALT_LEFT: string;
    ALT_RIGHT: string;
    META_LEFT: string;
    META_RIGHT: string;
    BACKSPACE: string;
    DELETE: string;
    ENTER: string;
    ESCAPE: string;
    TAB: string;
    COMMA: string;
    LESS_THAN: string;
    PERIOD: string;
    GREATER_THAN: string;
    SLASH: string;
    QUESTION: string;
    SEMICOLON: string;
    COLON: string;
    QUOTE: string;
    DOUBLE_QUOTE: string;
    BACKQUOTE: string;
    EXCLAIM: string;
    AT: string;
    HASH: string;
    DOLLAR: string;
    PERCENT: string;
    CARET: string;
    AMPERSAND: string;
    ASTERISK: string;
    LEFT_PAREN: string;
    RIGHT_PAREN: string;
    UNDERSCORE: string;
    PLUS: string;
    BRACKET_LEFT: string;
    BRACKET_RIGHT: string;
    BRACE_LEFT: string;
    BRACE_RIGHT: string;
    BACKSLASH: string;
    PIPE: string;
    MINUS: string;
    EQUALS: string;
    SPACE: string;
};
export const DOM_KEYBOARD_CODE_TO_LOGICAL_CODE: {
    KeyA: string;
    KeyB: string;
    KeyC: string;
    KeyD: string;
    KeyE: string;
    KeyF: string;
    KeyG: string;
    KeyH: string;
    KeyI: string;
    KeyJ: string;
    KeyK: string;
    KeyL: string;
    KeyM: string;
    KeyN: string;
    KeyO: string;
    KeyP: string;
    KeyQ: string;
    KeyR: string;
    KeyS: string;
    KeyT: string;
    KeyU: string;
    KeyV: string;
    KeyW: string;
    KeyX: string;
    KeyY: string;
    KeyZ: string;
    Digit0: string;
    Digit1: string;
    Digit2: string;
    Digit3: string;
    Digit4: string;
    Digit5: string;
    Digit6: string;
    Digit7: string;
    Digit8: string;
    Digit9: string;
    ArrowLeft: string;
    ArrowRight: string;
    ArrowUp: string;
    ArrowDown: string;
    ShiftLeft: string;
    ShiftRight: string;
    ControlLeft: string;
    ControlRight: string;
    AltLeft: string;
    AltRight: string;
    MetaLeft: string;
    MetaRight: string;
    Backspace: string;
    Delete: string;
    Enter: string;
    Escape: string;
    Tab: string;
    Comma: string;
    LessThan: string;
    Period: string;
    GreaterThan: string;
    Slash: string;
    Question: string;
    Semicolon: string;
    Colon: string;
    Quote: string;
    DoubleQuote: string;
    Backquote: string;
    Exclaim: string;
    At: string;
    Hash: string;
    Dollar: string;
    Percent: string;
    Caret: string;
    Ampersand: string;
    Asterisk: string;
    LeftParen: string;
    RightParen: string;
    Underscore: string;
    Plus: string;
    BracketLeft: string;
    BracketRight: string;
    BraceLeft: string;
    BraceRight: string;
    Backslash: string;
    Pipe: string;
    Minus: string;
    Equals: string;
    Space: string;
};
export const DOM_KEYBOARD_KEY_TO_LOGICAL_KEY: {
    a: string;
    b: string;
    c: string;
    d: string;
    e: string;
    f: string;
    g: string;
    h: string;
    i: string;
    j: string;
    k: string;
    l: string;
    m: string;
    n: string;
    o: string;
    p: string;
    q: string;
    r: string;
    s: string;
    t: string;
    u: string;
    v: string;
    w: string;
    x: string;
    y: string;
    z: string;
    A: string;
    B: string;
    C: string;
    D: string;
    E: string;
    F: string;
    G: string;
    H: string;
    I: string;
    J: string;
    K: string;
    L: string;
    M: string;
    N: string;
    O: string;
    P: string;
    Q: string;
    R: string;
    S: string;
    T: string;
    U: string;
    V: string;
    W: string;
    X: string;
    Y: string;
    Z: string;
    '0': string;
    '1': string;
    '2': string;
    '3': string;
    '4': string;
    '5': string;
    '6': string;
    '7': string;
    '8': string;
    '9': string;
    ',': string;
    '<': string;
    '.': string;
    '>': string;
    '/': string;
    '?': string;
    ';': string;
    ':': string;
    '\'': string;
    '"': string;
    '`': string;
    '!': string;
    '@': string;
    '#': string;
    $: string;
    '%': string;
    '^': string;
    '&': string;
    '*': string;
    '(': string;
    ')': string;
    _: string;
    '+': string;
    '[': string;
    ']': string;
    '{': string;
    '}': string;
    '\\': string;
    '|': string;
    '-': string;
    '=': string;
    ' ': string;
};
export function log(...args: any[]): void;
export function rect_from_pos_size(point: Point, size: Size): Rect;
export interface SurfaceContext {
    size(): Size;
    root(): View;
    set_root(root: View): any;
    repaint(): any;
    clear(): any;
    fill(rect: Rect, color: string): any;
    stroke(rect: Rect, color: string): any;
    fillBackgroundSize(size: Size, color: string): any;
    strokeBackgroundSize(size: Size, color: string): any;
    measureText(caption: string, font_name?: string): Size;
    fillStandardText(caption: string, x: number, y: number, font_name?: string, scale?: number): any;
    fillText(caption: string, pt: Point, color: string, font_name?: string, scale?: number): any;
    draw_glyph(codepoint: number, x: number, y: number, font_name: string, fill: string, scale?: number): any;
    set_sprite_scale(scale: number): any;
    set_smooth_sprites(sprite_smoothing: boolean): any;
    draw_sprite(pt: Point, sprite: Sprite): any;
    keyboard_focus(): View;
    set_keyboard_focus(view: View): any;
    is_keyboard_focus(view: View): any;
    release_keyboard_focus(view: View): any;
    view_to_local(pt: Point, view: View): Point;
    find_by_name(name: string): View | null;
}
export class CanvasSurface implements SurfaceContext {
    size(): Size;
    ctx: CanvasRenderingContext2D;
    debug: boolean;
    protected _input_callback: Callback;
    protected _keyboard_focus: View;
    constructor(w: number, h: number, scale?: number);
    addToPage(): void;
    set_root(root: View): void;
    root(): View;
    repaint(): void;
    clear(): void;
    fill(rect: Rect, color: string): void;
    stroke(rect: Rect, color: string): void;
    fillBackgroundSize(size: Size, color: string): void;
    strokeBackgroundSize(size: Size, color: string): void;
    fillRect(x: number, y: number, w: number, h: number, color: string): void;
    keyboard_focus(): View;
    set_keyboard_focus(view: View): void;
    is_keyboard_focus(view: View): boolean;
    release_keyboard_focus(view: View): void;
    on_input(cb: Callback): void;
    measureText(caption: string, font_name?: string): Size;
    fillStandardText(caption: string, x: number, y: number, font_name?: string, scale?: number): void;
    fillText(caption: string, pt: Point, color: string, font_name?: string, scale?: number): void;
    draw_glyph(codepoint: number, x: number, y: number, font_name: string, fill: string, scale?: number): void;
    load_jsonfont(basefont_data: any, name: string, ref_name: string): void;
    set_sprite_scale(scale: number): void;
    set_smooth_sprites(sprite_smoothing: boolean): void;
    draw_sprite(pt: Point, sprite: Sprite): void;
    draw_tilemap(tilemap: Tilemap, sheet: Sheet, x: number, y: number, scale: number): void;
    find_by_name(name: string): View | null;
    local_to_view(pt: Point, view: View): Point;
    view_to_local(pt: Point, view: View): Point;
    start(): void;
    start_input(): void;
    start_game(): void;
}
export class LayerView extends BaseParentView {
    _type: string;
    constructor(name?: string);
    draw(g: SurfaceContext): void;
    layout(g: SurfaceContext, available: Size): Size;
    set_visible(visible: boolean): void;
}
export type VAlign = "top" | "center" | "bottom" | "stretch";
export class HBox extends BaseParentView {
    pad: number;
    constructor();
    set_fill(fill: string): void;
    set_valign(valign: VAlign): void;
    layout(g: SurfaceContext, real_available: Size): Size;
    draw(g: SurfaceContext): void;
}
export type HAlign = "left" | "center" | "right" | "stretch";
export class VBox extends BaseParentView {
    pad: number;
    halign: HAlign;
    constructor();
    fill(): string;
    set_fill(fill: string): void;
    layout(g: SurfaceContext, real_available: Size): Size;
    draw(g: SurfaceContext): void;
    clear_children(): void;
}
export class GrowPanel extends BaseParentView {
    constructor();
    layout(g: SurfaceContext, available: Size): Size;
    draw(g: SurfaceContext): void;
    set_fill(fill: string): void;
    with_fill(fill: string): this;
}
export class ScrollView extends BaseParentView {
    constructor();
    draw(g: SurfaceContext): void;
    set_pref_width(num: number): void;
    layout(g: SurfaceContext, available: Size): Size;
    set_content(view: View): void;
}
export class PopupContainer extends BaseParentView {
    constructor();
    draw(g: SurfaceContext): void;
    layout(g: SurfaceContext, available: Size): Size;
    open_at(position: Point): void;
    hide(): void;
}
export class PopupLayer extends LayerView {
    constructor();
    draw(g: SurfaceContext): void;
    input(event: CoolEvent): void;
    can_receive_mouse(): boolean;
}
export class DialogLayer extends LayerView {
    constructor();
    draw(g: SurfaceContext): void;
}
export class DialogContainer extends BaseParentView {
    constructor();
    draw(g: SurfaceContext): void;
    layout(g: SurfaceContext, available: Size): Size;
}
export class KeystrokeCaptureView extends LayerView {
    constructor(main_view: View);
    input(event: CoolEvent): void;
}
export class FillChildPanel extends BaseParentView {
    constructor();
    layout(g: SurfaceContext, available: Size): Size;
    set_child(view: View): void;
    clear_children(): void;
}
export class TabbedPanel extends BaseParentView {
    constructor();
    draw(g: SurfaceContext): void;
    layout(g: SurfaceContext, available: Size): Size;
    add_view(title: string, content: View): void;
}
export class Label extends BaseView {
    protected _caption: string;
    constructor(caption?: string);
    caption(): string;
    set_caption(caption: string): void;
    draw(g: SurfaceContext): void;
    layout(g: SurfaceContext, available: Size): Size;
}
export class CustomLabel extends Label {
    constructor(text: string, cb: Callback);
    draw(ctx: SurfaceContext): void;
}
export class ActionButton extends BaseView {
    protected _caption: string;
    constructor(props?: any);
    caption(): string;
    set_caption(caption: string): void;
    draw(g: SurfaceContext): void;
    input(event: CoolEvent): void;
    layout(g: SurfaceContext, available: Size): Size;
}
export abstract class BaseSelectButton extends BaseView {
    _caption: string;
    _selected: boolean;
    _active: boolean;
    selected_icon: number;
    icon: number;
    constructor();
    selected(): boolean;
    set_selected(sel: boolean): void;
    caption(): string;
    set_caption(caption: string): void;
    draw(g: SurfaceContext): void;
    input(event: CoolEvent): void;
    layout(g: SurfaceContext, available: Size): Size;
}
export class ToggleButton extends BaseSelectButton {
    constructor(caption?: string);
    draw(ctx: SurfaceContext): void;
}
export class CheckButton extends BaseSelectButton {
    constructor();
}
export class RadioButton extends BaseSelectButton {
    constructor();
}
export class IconButton extends BaseView {
    constructor();
    draw(g: SurfaceContext): void;
    input(event: CoolEvent): void;
    layout(g: SurfaceContext, available: Size): Size;
    icon(): number;
    set_icon(icon: number): void;
}
export class SelectList extends BaseView {
    constructor(data: any[], renderer: any);
    draw(g: SurfaceContext): void;
    input(event: CoolEvent): void;
    set_data(data: any[]): void;
    layout(g: SurfaceContext, available: Size): Size;
}
export class Header extends BaseView {
    constructor(caption?: string);
    caption(): string;
    set_caption(caption: string): void;
    draw(g: SurfaceContext): void;
    layout(g: SurfaceContext, available: Size): Size;
}
export class HSpacer extends BaseView {
    constructor();
    layout(g: SurfaceContext, available: Size): Size;
    draw(g: SurfaceContext): void;
}
export class FontIcon extends BaseView {
    constructor(codepoint: number);
    draw(g: SurfaceContext): void;
    layout(g: SurfaceContext, available: Size): Size;
}
export class DropdownButton extends ActionButton {
    actions: Action[];
    constructor();
    set_actions(actions: Action[]): void;
}
type EventCategory = string;
type EventType = string;
type EventDirection = "down" | "up";
export class CoolEvent {
    type: EventType;
    category: EventCategory;
    timestamp: number;
    details: any;
    ctx: SurfaceContext;
    target: any;
    direction: EventDirection;
    stopped: any;
    constructor(ctx: SurfaceContext, category: EventCategory, type: EventType);
}
export type Modifiers = {
    shift: boolean;
    alt: boolean;
    ctrl: boolean;
    meta: boolean;
};
export const POINTER_CATEGORY: EventCategory;
export const POINTER_MOVE: EventType;
export const POINTER_DRAG: EventType;
export const POINTER_DOWN: EventType;
export const POINTER_UP: EventType;
export const PRIMARY_BUTTON: number;
export const SECONDARY_BUTTON: number;
export function gen_id(prefix: string): string;
export type Callback = (any: any) => any;
export class SuperArray {
    constructor();
    clear(): void;
    push_end(value: any): void;
    length(): number;
    pop_start(): any;
    forEach(cb: Callback): void;
}
export class Point {
    x: number;
    y: number;
    constructor(x: any, y: any);
    translate(x: number, y: number): Point;
    divide_floor(scale: number): Point;
    add(pt: Point): Point;
    set(x: number, y: number): void;
    copy_from(pt: Point): void;
    clone(): Point;
    subtract(trans: Point): Point;
    magnitude(): number;
    divide(val: number): Point;
    scale(val: number): Point;
    unit(): Point;
    constrain(bounds: Rect): Point;
    toString(): String;
}
export class Rect {
    x: number;
    y: number;
    w: number;
    h: number;
    constructor(x: any, y: any, w: any, h: any);
    clone(): Rect;
    contains(pt: Point): boolean;
    bottom(): number;
    right(): number;
    shrink(v: number): Rect;
    position(): Point;
    size(): Size;
    center(): Point;
    add_position(pt: Point): void;
    subtract(pt: Point): Rect;
    intersects(rect: Rect): boolean;
    contains_rect(rect: Rect): boolean;
}
export class Size {
    w: number;
    h: number;
    constructor(w: any, h: any);
    shrink(pad: number): Size;
    grow(pad: number): Size;
    subtract(delta: Point): Size;
    add(delta: Point): Size;
    contains(cursor: Point): boolean;
    toString(): String;
    equals(s: Size): boolean;
    clone(): Size;
}
export class PointerEvent extends CoolEvent {
    position: Point;
    delta: Point;
    button: number;
    modifiers: Modifiers;
    constructor(ctx: SurfaceContext, type: EventType, position: Point, delta: Point);
}
export const KEYBOARD_CATEGORY: EventCategory;
export const KEYBOARD_DOWN: EventType;
export const KEYBOARD_UP: EventType;
export type KeyCode = string;
export class KeyboardEvent extends CoolEvent {
    key: string;
    code: KeyCode;
    modifiers: Modifiers;
    constructor(surface: SurfaceContext, type: EventType);
}
export const SCROLL_CATEGORY: EventCategory;
export const SCROLL_EVENT: EventType;
export class ScrollEvent extends CoolEvent {
    delta: Point;
    position: Point;
    modifiers: Modifiers;
    constructor(surface: SurfaceContext, type: EventType, position: Point, delta: Point);
}
export const FOCUS_CATEGORY: EventCategory;
export const FOCUS_GAINED: EventType;
export const FOCUS_LOST: EventType;
export class FocusEvent extends CoolEvent {
    constructor(surface: SurfaceContext, FOCUS_GAINED: EventType);
}
export const COMMAND_CATEGORY: EventCategory;
export const COMMAND_ACTION: EventType;
export class CommandEvent extends CoolEvent {
    constructor(ctx: SurfaceContext, type: EventType, target: any);
}
export const COMMAND_CHANGE: EventType;
export type Action = {
    caption: string;
};
export const CLIPBOARD_CATEGORY: EventCategory;
export const CLIPBOARD_COPY: EventType;
export const CLIPBOARD_CUT: EventType;
export const CLIPBOARD_PASTE: EventType;
export class ClipboardEvent extends CoolEvent {
    content: any;
    mimetype: string;
}
export interface View {
    hflex(): boolean;
    vflex(): boolean;
    size(): Size;
    set_size(size: Size): any;
    position(): Point;
    set_position(point: Point): any;
    layout(g: SurfaceContext, available: Size): Size;
    draw(g: SurfaceContext): void;
    visible(): boolean;
    input(event: CoolEvent): void;
    on(type: string, cb: Callback): void;
    off(type: string, cb: Callback): void;
    name(): string;
}
export interface ParentView {
    is_parent_view(): boolean;
    get_children(): View[];
    find_child(id: string): View | null;
    clip_children(): boolean;
    can_receive_mouse(): boolean;
}
export abstract class BaseParentView implements View, ParentView {
    protected _hflex: boolean;
    protected _vflex: boolean;
    id: string;
    protected _visible: boolean;
    protected _children: View[];
    protected _name: string;
    constructor(id: string);
    hflex(): boolean;
    set_hflex(hflex: any): void;
    vflex(): boolean;
    set_vflex(vflex: any): void;
    protected log(...args: any[]): void;
    size(): Size;
    set_size(size: Size): void;
    position(): Point;
    set_position(point: Point): void;
    clip_children(): boolean;
    draw(g: SurfaceContext): void;
    get_children(): View[];
    find_child(id: string): View | null;
    add(view: View): void;
    remove(view: View): void;
    input(event: CoolEvent): void;
    is_parent_view(): boolean;
    name(): string;
    set_name(name: string): void;
    on(type: string, cb: Callback): void;
    off(type: string, cb: Callback): void;
    fire(type: string, payload: any): void;
    visible(): boolean;
    abstract layout(g: SurfaceContext, available: Size): Size;
    can_receive_mouse(): boolean;
}
export abstract class BaseView implements View {
    protected _hflex: boolean;
    protected _vflex: boolean;
    protected _visible: boolean;
    protected _name: string;
    protected id: string;
    constructor(id: string);
    hflex(): boolean;
    vflex(): boolean;
    protected log(...args: any[]): void;
    on(type: string, cb: Callback): void;
    off(type: string, cb: Callback): void;
    fire(type: string, payload: any): void;
    size(): Size;
    set_size(size: Size): void;
    position(): Point;
    set_position(point: Point): void;
    input(event: CoolEvent): void;
    name(): string;
    set_name(name: string): void;
    visible(): boolean;
    set_visible(visible: boolean): void;
    abstract layout(g: SurfaceContext, available: Size): Size;
    abstract draw(g: SurfaceContext): void;
}
export function with_props(comp: View, json: any): View;
export function with_action(view: ActionButton, cb: Callback): View;
export function with_change(view: BaseSelectButton, cb: Callback): View;
export function randi(min: number, max: number): number;
export class DebugLensGlass extends BaseView {
    constructor(p: DebugLens, size: Size);
    layout(g: CanvasSurface, available: Size): Size;
    draw(g: CanvasSurface): void;
    draw_names(): boolean;
    set_draw_names(selected: boolean): void;
    set_draw_sizes(selected: boolean): void;
    set_draw_bounds(selected: boolean): void;
    set_draw_flex(selected: boolean): void;
    set_draw_align(selected: boolean): void;
    input(event: CoolEvent): void;
}
export class ResizeHandle extends BaseView {
    constructor(lens: DebugLens);
    input(event: CoolEvent): void;
    draw(g: CanvasSurface): void;
    layout(g: CanvasSurface, available: Size): Size;
}
export class DebugLens extends BaseParentView {
    _selected: View | null;
    constructor();
    input(event: CoolEvent): void;
    can_receive_mouse(): boolean;
    draw(g: CanvasSurface): void;
    layout(g: CanvasSurface, available: Size): Size;
    set_visible(b: boolean): void;
    set_selected(view: View | null): void;
}
export class DebugLayer extends LayerView {
    constructor();
    layout(g: CanvasSurface, available: Size): Size;
}
export class TableView extends BaseParentView {
    data: any[];
    columns_keys: string[];
    columns_widths: number[];
    constructor(songs: any[], columns_keys: string[], columns_widths: number[]);
    draw(g: CanvasSurface): void;
    layout(g: CanvasSurface, available: Size): Size;
}
export class TextLine extends BaseView {
    text: string;
    pref_width: number;
    constructor();
    draw(g: SurfaceContext): void;
    input(event: CoolEvent): void;
    layout(g: SurfaceContext, available: Size): Size;
    set_text(name: string): void;
    set_pref_width(w: number): void;
}
type IndexPosition = {
    count: number;
};
type RCPosition = {
    block: number;
    inset: number;
};
declare class DocModel {
    _full_text: string;
    blocks: string[];
    cursor: IndexPosition;
    constructor();
    set_text(text: string): void;
    index_to_rc(index: IndexPosition): RCPosition;
    rc_to_index(rc: RCPosition): IndexPosition;
    char_at_index(n: IndexPosition): string;
    char_at_rc(rc: RCPosition): string;
    add_rc(rc: RCPosition, count: number): RCPosition;
    subtract_rc(rc: RCPosition, count: number): RCPosition;
    insert(key: any): void;
    delete_right(count: number): void;
    delete_left(count: number): void;
    cursor_left(count: number): void;
    cursor_right(count: number): void;
    cursor_up(count: number): void;
    cursor_down(count: number): void;
    cursor_line_start(): void;
    cursor_line_end(): void;
    split_block(rc: RCPosition): void;
    set_cursor_rc(rc1: RCPosition): void;
    get_cursor_index(): IndexPosition;
}
declare class DocLayout {
    model: DocModel;
    blocks: any[];
    constructor(doc: DocModel);
    size: Size;
}
export class TextBox extends BaseView {
    _doc: DocModel;
    render_tree_root: DocLayout;
    constructor();
    set_pref_height(h: number): void;
    draw(g: SurfaceContext): void;
    layout(g: SurfaceContext, available: Size): Size;
    input(event: CoolEvent): void;
    set_text(text: string): void;
}
export class NumberTextLine extends HBox {
    constructor();
    draw(g: any): void;
    set_value(value: number): void;
    value(): number;
}
export const BASE_FONT: {
    version: number;
    name: string;
    color_palette: string[];
    sheets: {
        clazz: string;
        id: string;
        name: string;
        sprites: {
            clazz: string;
            id: string;
            name: string;
            w: number;
            h: number;
            data: number[];
        }[];
    }[];
    fonts: {
        clazz: string;
        id: string;
        name: string;
        glyphs: {
            clazz: string;
            id: string;
            name: string;
            w: number;
            h: number;
            data: number[];
            meta: {
                codepoint: number;
                left: number;
                right: number;
                baseline: number;
            };
        }[];
    }[];
    maps: {
        clazz: string;
        id: string;
        name: string;
        w: number;
        h: number;
        data: (string | number)[];
    }[];
};

//# sourceMappingURL=types.d.ts.map
